@@ -41,6 +41,12 @@ static StorageClassInfo const storageClasses[] = {
 	{C_SPEC_EXTERN,
 		C_KIND_GLOBAL_NAME | C_KIND_LOCAL_NAME | C_KIND_FUNCTION,
 		"extern"},
+	{C_SPEC_THREAD_SPECIFIC | C_SPEC_STATIC,
+		C_KIND_GLOBAL_NAME | C_KIND_LOCAL_NAME,
+		"static __declspec(thread)"},
+	{C_SPEC_THREAD_SPECIFIC,
+		C_KIND_GLOBAL_NAME,
+		"__declspec(thread)"},
 	{C_SPEC_STATIC | C_SPEC_INLINE,
 		C_KIND_FUNCTION,
 		"static inline"},
@@ -200,7 +206,7 @@ CDeclSpec CDeclSpecCombine(CDeclSpec spec1, CDeclSpec spec2)
 					switch((spec1.specifiers | spec2.specifiers)
 							& (C_SPEC_SIGNED | C_SPEC_UNSIGNED |
 							   C_SPEC_SHORT | C_SPEC_LONG |
-							   C_SPEC_LONG_LONG | C_SPEC_NATIVE))
+							   C_SPEC_LONG_LONG))
 					{
 						case C_SPEC_SIGNED:
 						{
@@ -216,27 +222,6 @@ CDeclSpec CDeclSpecCombine(CDeclSpec spec1, CDeclSpec spec2)
 						}
 						break;
 
-						case C_SPEC_NATIVE:
-						{
-							result.baseType = ILType_Int;
-							okSpecifiers = C_SPEC_NATIVE;
-						}
-						break;
-
-						case C_SPEC_SIGNED | C_SPEC_NATIVE:
-						{
-							result.baseType = ILType_Int;
-							okSpecifiers = C_SPEC_SIGNED | C_SPEC_NATIVE;
-						}
-						break;
-
-						case C_SPEC_UNSIGNED | C_SPEC_NATIVE:
-						{
-							result.baseType = ILType_UInt;
-							okSpecifiers = C_SPEC_UNSIGNED | C_SPEC_NATIVE;
-						}
-						break;
-
 						case C_SPEC_SHORT:
 						case C_SPEC_SIGNED | C_SPEC_SHORT:
 						{
@@ -255,28 +240,14 @@ CDeclSpec CDeclSpecCombine(CDeclSpec spec1, CDeclSpec spec2)
 						case C_SPEC_LONG:
 						case C_SPEC_SIGNED | C_SPEC_LONG:
 						{
-							if(CTypeLongSize == 4)
-							{
-								result.baseType = ILType_Int32;
-							}
-							else
-							{
-								result.baseType = ILType_Int64;
-							}
+							result.baseType = ILType_Int;
 							okSpecifiers = C_SPEC_SIGNED | C_SPEC_LONG;
 						}
 						break;
 
 						case C_SPEC_UNSIGNED | C_SPEC_LONG:
 						{
-							if(CTypeLongSize == 4)
-							{
-								result.baseType = ILType_UInt32;
-							}
-							else
-							{
-								result.baseType = ILType_UInt64;
-							}
+							result.baseType = ILType_UInt;
 							okSpecifiers = C_SPEC_UNSIGNED | C_SPEC_LONG;
 						}
 						break;
@@ -308,8 +279,7 @@ CDeclSpec CDeclSpecCombine(CDeclSpec spec1, CDeclSpec spec2)
 				{
 					/* Look for "short", "long", or "long long" */
 					switch((spec1.specifiers | spec2.specifiers)
-							& (C_SPEC_SHORT | C_SPEC_LONG |
-							   C_SPEC_LONG_LONG | C_SPEC_NATIVE))
+							& (C_SPEC_SHORT | C_SPEC_LONG | C_SPEC_LONG_LONG))
 					{
 						case C_SPEC_SHORT:
 						{
@@ -318,23 +288,9 @@ CDeclSpec CDeclSpecCombine(CDeclSpec spec1, CDeclSpec spec2)
 						}
 						break;
 
-						case C_SPEC_NATIVE:
-						{
-							result.baseType = ILType_UInt;
-							okSpecifiers = C_SPEC_NATIVE | C_SPEC_UNSIGNED;
-						}
-						break;
-
 						case C_SPEC_LONG:
 						{
-							if(CTypeLongSize == 4)
-							{
-								result.baseType = ILType_UInt32;
-							}
-							else
-							{
-								result.baseType = ILType_UInt64;
-							}
+							result.baseType = ILType_UInt;
 							okSpecifiers = C_SPEC_LONG | C_SPEC_UNSIGNED;
 						}
 						break;
@@ -400,30 +356,79 @@ CDeclSpec CDeclSpecCombine(CDeclSpec spec1, CDeclSpec spec2)
 
 				case IL_META_ELEMTYPE_I:
 				{
-					/* Look for "signed" and "unsigned" */
+					/* Look for "long long", "signed", and "unsigned" */
 					if(((spec1.specifiers | spec2.specifiers)
-							& C_SPEC_UNSIGNED) != 0)
+							& C_SPEC_LONG_LONG) != 0)
+					{
+						if(((spec1.specifiers | spec2.specifiers)
+								& C_SPEC_UNSIGNED) != 0)
+						{
+							result.baseType = ILType_UInt64;
+							okSpecifiers = C_SPEC_UNSIGNED | C_SPEC_LONG |
+										   C_SPEC_LONG_LONG;
+						}
+						else if(((spec1.specifiers | spec2.specifiers)
+									& C_SPEC_SIGNED) != 0)
+						{
+							result.baseType = ILType_Int64;
+							okSpecifiers = C_SPEC_SIGNED | C_SPEC_LONG |
+										   C_SPEC_LONG_LONG;
+						}
+						else
+						{
+							result.baseType = ILType_Int64;
+							okSpecifiers = C_SPEC_LONG | C_SPEC_LONG_LONG;
+						}
+					}
+					else if(((spec1.specifiers | spec2.specifiers)
+								& C_SPEC_UNSIGNED) != 0)
 					{
 						result.baseType = ILType_UInt;
-						okSpecifiers = C_SPEC_UNSIGNED | C_SPEC_NATIVE;
+						okSpecifiers = C_SPEC_UNSIGNED | C_SPEC_LONG;
 					}
 					else if(((spec1.specifiers | spec2.specifiers)
 								& C_SPEC_SIGNED) != 0)
 					{
 						result.baseType = ILType_Int;
-						okSpecifiers = C_SPEC_SIGNED | C_SPEC_NATIVE;
+						okSpecifiers = C_SPEC_SIGNED | C_SPEC_LONG;
+					}
+					else
+					{
+						okSpecifiers = C_SPEC_LONG | C_SPEC_SIGNED |
+									   C_SPEC_UNSIGNED;
 					}
 				}
 				break;
 
 				case IL_META_ELEMTYPE_U:
 				{
-					/* Look for "unsigned" */
+					/* Look for "long long" and "unsigned" */
 					if(((spec1.specifiers | spec2.specifiers)
-							& C_SPEC_UNSIGNED) != 0)
+							& C_SPEC_LONG_LONG) != 0)
+					{
+						if(((spec1.specifiers | spec2.specifiers)
+									& C_SPEC_UNSIGNED) != 0)
+						{
+							result.baseType = ILType_UInt64;
+							okSpecifiers = C_SPEC_UNSIGNED | C_SPEC_LONG |
+										   C_SPEC_LONG_LONG;
+						}
+						else
+						{
+							result.baseType = ILType_UInt64;
+							okSpecifiers = C_SPEC_LONG | C_SPEC_UNSIGNED |
+										   C_SPEC_LONG_LONG;
+						}
+					}
+					else if(((spec1.specifiers | spec2.specifiers)
+								& C_SPEC_UNSIGNED) != 0)
 					{
 						result.baseType = ILType_UInt;
-						okSpecifiers = C_SPEC_UNSIGNED | C_SPEC_NATIVE;
+						okSpecifiers = C_SPEC_UNSIGNED | C_SPEC_LONG;
+					}
+					else
+					{
+						okSpecifiers = C_SPEC_LONG | C_SPEC_UNSIGNED;
 					}
 				}
 				break;
@@ -435,7 +440,7 @@ CDeclSpec CDeclSpecCombine(CDeclSpec spec1, CDeclSpec spec2)
 							& C_SPEC_LONG_LONG) != 0)
 					{
 						/* "long long float" == "long double" */
-						result.baseType = ILType_Float;
+						result.baseType = ILType_Float64;
 						okSpecifiers = C_SPEC_LONG | C_SPEC_LONG_LONG;
 					}
 					else if(((spec1.specifiers | spec2.specifiers)
@@ -454,7 +459,7 @@ CDeclSpec CDeclSpecCombine(CDeclSpec spec1, CDeclSpec spec2)
 					if(((spec1.specifiers | spec2.specifiers)
 							& (C_SPEC_LONG | C_SPEC_LONG_LONG)) != 0)
 					{
-						result.baseType = ILType_Float;
+						result.baseType = ILType_Float64;
 						okSpecifiers = C_SPEC_LONG | C_SPEC_LONG_LONG;
 					}
 				}
@@ -467,7 +472,7 @@ CDeclSpec CDeclSpecCombine(CDeclSpec spec1, CDeclSpec spec2)
 							& (C_SPEC_LONG | C_SPEC_LONG_LONG)) != 0)
 					{
 						/* "long long double" == "long double" */
-						result.baseType = ILType_Float;
+						result.baseType = ILType_Float64;
 						okSpecifiers = C_SPEC_LONG | C_SPEC_LONG_LONG;
 					}
 				}
@@ -502,11 +507,10 @@ CDeclSpec CDeclSpecCombine(CDeclSpec spec1, CDeclSpec spec2)
 		result.dupSpecifiers |= C_SPEC_LONG_AND_SHORT;
 	}
 
-	/* Check for duplicate "signed", "unsigned", "short", and "__native__"
-	   but don't worry about "long" as we max out the sizes above */
+	/* Check for duplicate "signed", "unsigned", and "short" but don't
+	   worry about "long" as we max out the sizes above */
 	result.dupSpecifiers |= (spec1.specifiers & spec2.specifiers &
-								(C_SPEC_SIGNED | C_SPEC_UNSIGNED |
-								 C_SPEC_SHORT | C_SPEC_NATIVE));
+						(C_SPEC_SIGNED | C_SPEC_UNSIGNED | C_SPEC_SHORT));
 
 	/* Done */
 	return result;
@@ -666,7 +670,9 @@ CDeclSpec CDeclSpecFinalize(CDeclSpec spec, ILNode *node,
 	ReportDuplicate(node, spec.dupSpecifiers, C_SPEC_SIGNED, "signed");
 	ReportDuplicate(node, spec.dupSpecifiers, C_SPEC_UNSIGNED, "unsigned");
 	ReportDuplicate(node, spec.dupSpecifiers, C_SPEC_SHORT, "short");
-	ReportDuplicate(node, spec.dupSpecifiers, C_SPEC_NATIVE, "__native__");
+	ReportDuplicate(node, spec.dupSpecifiers, C_SPEC_THREAD_SPECIFIC,
+					"__declspec(thread)");
+	ReportDuplicate(node, spec.dupSpecifiers, C_SPEC_BOX, "__box");
 
 	/* Copy the common type qualifiers to "result.specifiers" */
 	result.specifiers |= (spec.specifiers & C_SPEC_TYPE_COMMON);
@@ -684,13 +690,8 @@ CDeclSpec CDeclSpecFinalize(CDeclSpec spec, ILNode *node,
 			{
 				result.baseType = ILType_Int16;
 			}
-			if((spec.specifiers & C_SPEC_NATIVE) != 0)
-			{
-				spec.dupSpecifiers |= C_SPEC_INVALID_COMBO;
-			}
 		}
-		else if((spec.specifiers & C_SPEC_LONG_LONG) != 0 &&
-				CTypeLongSize == 4)
+		else if((spec.specifiers & C_SPEC_LONG_LONG) != 0)
 		{
 			if((spec.specifiers & C_SPEC_UNSIGNED) != 0)
 			{
@@ -700,42 +701,8 @@ CDeclSpec CDeclSpecFinalize(CDeclSpec spec, ILNode *node,
 			{
 				result.baseType = ILType_Int64;
 			}
-			if((spec.specifiers & C_SPEC_NATIVE) != 0)
-			{
-				spec.dupSpecifiers |= C_SPEC_INVALID_COMBO;
-			}
 		}
-		else if((spec.specifiers & C_SPEC_LONG) != 0 && CTypeLongSize == 4)
-		{
-			if((spec.specifiers & C_SPEC_UNSIGNED) != 0)
-			{
-				result.baseType = ILType_UInt32;
-			}
-			else
-			{
-				result.baseType = ILType_Int32;
-			}
-			if((spec.specifiers & C_SPEC_NATIVE) != 0)
-			{
-				spec.dupSpecifiers |= C_SPEC_INVALID_COMBO;
-			}
-		}
-		else if((spec.specifiers & (C_SPEC_LONG | C_SPEC_LONG_LONG)) != 0)
-		{
-			if((spec.specifiers & C_SPEC_UNSIGNED) != 0)
-			{
-				result.baseType = ILType_UInt64;
-			}
-			else
-			{
-				result.baseType = ILType_Int64;
-			}
-			if((spec.specifiers & C_SPEC_NATIVE) != 0)
-			{
-				spec.dupSpecifiers |= C_SPEC_INVALID_COMBO;
-			}
-		}
-		else if((spec.specifiers & C_SPEC_NATIVE) != 0)
+		else if((spec.specifiers & C_SPEC_LONG) != 0)
 		{
 			if((spec.specifiers & C_SPEC_UNSIGNED) != 0)
 			{
@@ -767,6 +734,26 @@ CDeclSpec CDeclSpecFinalize(CDeclSpec spec, ILNode *node,
 		result.baseType = spec.baseType;
 	}
 
+	/* If "__box" is present, then convert value types into reference types */
+	if((spec.specifiers & C_SPEC_BOX) != 0)
+	{
+		if(ILTypeIsValue(result.baseType))
+		{
+			ILClass *classInfo = ILTypeToClass(&CCCodeGen, result.baseType);
+			if(!classInfo)
+			{
+				CCOutOfMemory();
+			}
+			result.baseType = ILType_FromClass(classInfo);
+		}
+		else
+		{
+			ReportError(node, name,
+						_("cannot use `__box' in declaration of `%s'"),
+						_("cannot use `__box' in declaration"));
+		}
+	}
+
 	/* Print pending errors that were detected by "CDeclSpecCombine" */
 	if((spec.dupSpecifiers & C_SPEC_MULTIPLE_BASES) != 0)
 	{
@@ -792,18 +779,20 @@ CDeclSpec CDeclSpecFinalize(CDeclSpec spec, ILNode *node,
 			== C_SPEC_INVALID_COMBO)
 	{
 		ReportError(node, name,
-			_("long, short, signed, unsigned or __native__ invalid for `%s'"),
-			_("long, short, signed, unsigned or __native__ invalid here"));
+			_("long, short, signed, or unsigned invalid for `%s'"),
+			_("long, short, signed, or unsigned invalid here"));
 	}
 
 	/* Return the result to the caller */
 	return result;
 }
 
-CDeclarator CDeclCreateOpenArray(ILGenInfo *info, CDeclarator elem)
+CDeclarator CDeclCreateOpenArray(ILGenInfo *info, CDeclarator elem,
+								 int gcSpecifier)
 {
 	CDeclarator result;
 	ILType *type;
+	ILType **hole;
 
 	/* Create a C#-style array reference, which we will turn into a
 	   C-style array type when "CDeclFinalize" is called.  For now,
@@ -814,6 +803,19 @@ CDeclarator CDeclCreateOpenArray(ILGenInfo *info, CDeclarator elem)
 		ILGenOutOfMemory(info);
 	}
 	ILTypeSetSize(type, 0, (long)(-1L));
+	hole = &(ILType_ElemType(type));
+
+	/* Mark the type with the GC specifier if necessary */
+	if(gcSpecifier == ILGC_Managed)
+	{
+		*hole = CTypeAddManaged(info, *hole);
+		hole = &((*hole)->un.modifier__.type__);
+	}
+	else if(gcSpecifier == ILGC_Unmanaged)
+	{
+		*hole = CTypeAddUnmanaged(info, *hole);
+		hole = &((*hole)->un.modifier__.type__);
+	}
 
 	/* Insert "type" into the hole in "elem" to create "result" */
 	result.name = elem.name;
@@ -821,10 +823,10 @@ CDeclarator CDeclCreateOpenArray(ILGenInfo *info, CDeclarator elem)
 	if(elem.typeHole != 0)
 	{
 		result.type = elem.type;
-		ILType_ElemType(type) = *(elem.typeHole);
+		*hole = *(elem.typeHole);
 		if(*(elem.typeHole) == ILType_Invalid)
 		{
-			result.typeHole = &(ILType_ElemType(type));
+			result.typeHole = hole;
 		}
 		else
 		{
@@ -835,7 +837,7 @@ CDeclarator CDeclCreateOpenArray(ILGenInfo *info, CDeclarator elem)
 	else
 	{
 		result.type = type;
-		result.typeHole = &(ILType_ElemType(type));
+		result.typeHole = hole;
 	}
 	result.isKR = 0;
 	result.params = elem.params;
@@ -843,10 +845,12 @@ CDeclarator CDeclCreateOpenArray(ILGenInfo *info, CDeclarator elem)
 	return result;
 }
 
-CDeclarator CDeclCreateArray(ILGenInfo *info, CDeclarator elem, ILUInt32 size)
+CDeclarator CDeclCreateArray(ILGenInfo *info, CDeclarator elem,
+							 ILNode *size, int gcSpecifier)
 {
 	CDeclarator result;
 	ILType *type;
+	ILType **hole;
 
 	/* Create a C#-style array reference, which we will turn into a
 	   C-style array type when "CDeclFinalize" is called.  For now,
@@ -856,13 +860,20 @@ CDeclarator CDeclCreateArray(ILGenInfo *info, CDeclarator elem, ILUInt32 size)
 	{
 		ILGenOutOfMemory(info);
 	}
-	if(((long)size) == (long)(-1L))
+	ILTypeSetSize(type, 0, (long)(ILNativeInt)size);
+	hole = &(ILType_ElemType(type));
+
+	/* Mark the type with the GC specifier if necessary */
+	if(gcSpecifier == ILGC_Managed)
 	{
-		/* Prevent confusion with "CDeclCreateOpenArray".  The size
-		   will eventually be rejected by "ReplaceArrayTypes" anyway */
-		--size;
+		*hole = CTypeAddManaged(info, *hole);
+		hole = &((*hole)->un.modifier__.type__);
+	}
+	else if(gcSpecifier == ILGC_Unmanaged)
+	{
+		*hole = CTypeAddUnmanaged(info, *hole);
+		hole = &((*hole)->un.modifier__.type__);
 	}
-	ILTypeSetSize(type, 0, (long)size);
 
 	/* Insert "type" into the hole in "elem" to create "result" */
 	result.name = elem.name;
@@ -870,10 +881,10 @@ CDeclarator CDeclCreateArray(ILGenInfo *info, CDeclarator elem, ILUInt32 size)
 	if(elem.typeHole != 0)
 	{
 		result.type = elem.type;
-		ILType_ElemType(type) = *(elem.typeHole);
+		*hole = *(elem.typeHole);
 		if(*(elem.typeHole) == ILType_Invalid)
 		{
-			result.typeHole = &(ILType_ElemType(type));
+			result.typeHole = hole;
 		}
 		else
 		{
@@ -884,7 +895,7 @@ CDeclarator CDeclCreateArray(ILGenInfo *info, CDeclarator elem, ILUInt32 size)
 	else
 	{
 		result.type = type;
-		result.typeHole = &(ILType_ElemType(type));
+		result.typeHole = hole;
 	}
 	result.isKR = 0;
 	result.params = elem.params;
@@ -938,6 +949,52 @@ CDeclarator CDeclCreatePointer(ILGenInfo *info, int qualifiers,
 	return result;
 }
 
+CDeclarator CDeclCreateByRef(ILGenInfo *info, int qualifiers,
+							 CDeclarator *refType)
+{
+	CDeclarator result;
+	ILType *type;
+
+	/* Create the actual reference type */
+	if(refType)
+	{
+		type = CTypeCreateByRef(info, refType->type);
+		result.typeHole = refType->typeHole;
+	}
+	else
+	{
+		type = CTypeCreateByRef(info, ILType_Invalid);
+		result.typeHole = &(ILType_Ref(type));
+	}
+
+	/* Wrap the reference type in the "const" and "volatile" qualifiers */
+	if((qualifiers & C_SPEC_CONST) != 0)
+	{
+		type = CTypeAddConst(info, type);
+	}
+	if((qualifiers & C_SPEC_VOLATILE) != 0)
+	{
+		type = CTypeAddVolatile(info, type);
+	}
+
+	/* Construct the return value (the type hole was set above) */
+	result.name = 0;
+	result.node = 0;
+	result.type = type;
+	result.isKR = 0;
+	if(refType)
+	{
+		result.params = refType->params;
+		result.attrs = refType->attrs;
+	}
+	else
+	{
+		result.params = 0;
+		result.attrs = 0;
+	}
+	return result;
+}
+
 /*
  * Print an error that warns about parameter redeclaration.
  */
@@ -1097,6 +1154,12 @@ static ILType *ParamsToSignature(ILGenInfo *info, ILNode *params,
 			/* Decay array types to their pointer forms */
 			type = CTypeDecay(info, type);
 
+			/* Pass complex types by pointer, not by value */
+			if(CTypeIsComplex(type))
+			{
+				type = CTypeCreateComplexPointer(info, type);
+			}
+
 			/* Add the new parameter to the function signature */
 			if(!ILTypeAddParam(info->context, signature, type))
 			{
@@ -1143,16 +1206,20 @@ CDeclarator CDeclCreatePrototype(ILGenInfo *info, CDeclarator decl,
 	ILType *signature;
 	ILType **returnHole;
 
-	/* Bail out if we already have parameters in the declarator */
+	/* Bail out if we already have parameters in the declarator
+	   and we aren't declaring a function pointer return value */
 	if(decl.params)
 	{
-		CCError(_("cannot declare a function returning a function"));
-		return decl;
+		if(!ILType_IsPointer(decl.type) || !(decl.typeHole))
+		{
+			CCError(_("cannot declare a function returning a function"));
+			return decl;
+		}
 	}
 
 	/* If the declarator has a type hole, then create a signature
 	   inside it, and replace the hole with the return type */
-	if(decl.typeHole)
+	if(decl.typeHole && !(decl.params))
 	{
 		signature = ParamsToSignature(info, params, 0, 0, &returnHole);
 		*(decl.typeHole) = signature;
@@ -1160,6 +1227,15 @@ CDeclarator CDeclCreatePrototype(ILGenInfo *info, CDeclarator decl,
 		return decl;
 	}
 
+	/* Deal with functions that return function pointers */
+	if(decl.params)
+	{
+		signature = ParamsToSignature(info, params, 0, 0, &returnHole);
+		*returnHole = decl.type;
+		decl.type = signature;
+		return decl;
+	}
+
 	/* Modify the declarator and return it */
 	if(params)
 	{
@@ -1202,8 +1278,9 @@ CDeclarator CDeclCombine(CDeclarator decl1, CDeclarator decl2)
  */
 static ILType *ReplaceArrayTypes(ILGenInfo *info, ILType *type)
 {
+	CTypeLayoutInfo layout;
 	ILType *elemType;
-	ILUInt32 elemSize;
+	char *name;
 
 	if(type != 0 && ILType_IsComplex(type))
 	{
@@ -1218,6 +1295,13 @@ static ILType *ReplaceArrayTypes(ILGenInfo *info, ILType *type)
 					type = ReplaceArrayTypes(info, ILType_Ref(type));
 					type = CTypeAddFunctionPtr(info, type);
 				}
+				else if(ILType_IsClass(ILType_Ref(type)))
+				{
+					/* Pointing to a class reference.  Remove the pointer.
+					   This turns types like "String *" into "String" for
+					   compatibility with Managed C++ */
+					type = ILType_Ref(type);
+				}
 				else
 				{
 					ILType_Ref(type) =
@@ -1228,22 +1312,60 @@ static ILType *ReplaceArrayTypes(ILGenInfo *info, ILType *type)
 
 			case IL_TYPE_COMPLEX_ARRAY:
 			{
-				/* Replace array types in the element type */
-				elemType = ReplaceArrayTypes(info, ILType_ElemType(type));
-
-				/* Report an error if the type has unknown or variable size */
-				elemSize = CTypeSizeAndAlign(elemType, 0);
-				if(elemSize == CTYPE_UNKNOWN)
+				/* Check for "__gc" and "__nogc" modifiers */
+				if(CTypeIsManaged(ILType_ElemType(type)))
 				{
-					char *name = CTypeToName(info, elemType);
-					CCError(_("storage size of `%s' is not known"), name);
-					ILFree(name);
-					elemType = ILType_Int32;
+					/* Turn an unmanaged array into its managed counterpart */
+					elemType = ReplaceArrayTypes
+						(info, CTypeStripGC(ILType_ElemType(type)));
+					if(ILTypeIsReference(elemType) || ILTypeIsValue(elemType))
+					{
+						goto managed;
+					} 
+					else
+					{
+						name = CTypeToName(info, elemType);
+						CCError(_("__gc used with unmanaged type `%s'"), name);
+						ILFree(name);
+					}
 				}
-				else if(elemSize == CTYPE_DYNAMIC)
+				else if(CTypeIsUnmanaged(ILType_ElemType(type)))
 				{
-					char *name = CTypeToName(info, elemType);
-					CCError(_("storage size of `%s' is not constant"), name);
+					/* We expect the element type to be unmanaged */
+					elemType = ReplaceArrayTypes
+						(info, CTypeStripGC(ILType_ElemType(type)));
+					if(ILTypeIsReference(elemType))
+					{
+						name = CTypeToName(info, elemType);
+						CCError(_("__nogc used with managed type `%s'"), name);
+						ILFree(name);
+						goto managed;
+					}
+				}
+				else
+				{
+					/* Determine the array type from the element type */
+					elemType = ReplaceArrayTypes(info, ILType_ElemType(type));
+					if(ILTypeIsReference(elemType))
+					{
+					managed:
+						if(ILType_Size(type) != (long)(-1))
+						{
+							CCError(_("cannot specify a size for managed "
+									  "array types"));
+						}
+						ILTypeSetSize(type, 0, 0);
+						ILType_ElemType(type) = elemType;
+						break;
+					}
+				}
+
+				/* Report an error if the type has unknown layout */
+				CTypeGetLayoutInfo(elemType, &layout);
+				if(layout.category == C_TYPECAT_NO_LAYOUT)
+				{
+					name = CTypeToName(info, elemType);
+					CCError(_("storage size of `%s' is not known"), name);
 					ILFree(name);
 					elemType = ILType_Int32;
 				}
@@ -1255,7 +1377,9 @@ static ILType *ReplaceArrayTypes(ILGenInfo *info, ILType *type)
 				}
 				else
 				{
-					type = CTypeCreateArray(info, elemType, ILType_Size(type));
+					type = CTypeCreateArrayNode
+						(info, elemType,
+						 (ILNode *)(ILNativeInt)ILType_Size(type));
 					if(!type)
 					{
 						CCError(_("size of array is too large"));
@@ -1282,6 +1406,12 @@ static ILType *ReplaceArrayTypes(ILGenInfo *info, ILType *type)
 				type->un.method__.retType__ =
 					CTypeDecay(info, ReplaceArrayTypes
 						(info, type->un.method__.retType__));
+				if(CTypeIsComplex(type->un.method__.retType__))
+				{
+					type->un.method__.retType__ =
+						CTypeCreateComplexPointer
+							(info, type->un.method__.retType__);
+				}
 			}
 			break;
 
@@ -1346,6 +1476,10 @@ ILType *CDeclFinalize(ILGenInfo *info, CDeclSpec spec, CDeclarator decl,
 		signature = ParamsToSignature(info, decl.params, declaredParams,
 									  method, &returnHole);
 		*returnHole = CTypeDecay(info, type);
+		if(CTypeIsComplex(*returnHole))
+		{
+			*returnHole = CTypeCreateComplexPointer(info, *returnHole);
+		}
 		type = signature;
 		ILMemberSetSignature((ILMember *)method, signature);
 		ILMethodSetCallConv(method, ILType_CallConv(signature));
@@ -1355,6 +1489,10 @@ ILType *CDeclFinalize(ILGenInfo *info, CDeclSpec spec, CDeclarator decl,
 		/* Create a function signature pointer type */
 		signature = ParamsToSignature(info, decl.params, 0, 0, &returnHole);
 		*returnHole = CTypeDecay(info, type);
+		if(CTypeIsComplex(*returnHole))
+		{
+			*returnHole = CTypeCreateComplexPointer(info, *returnHole);
+		}
 		type = signature;
 	}
 
