@@ -39,14 +39,15 @@ extern	"C" {
 #define	C_SPEC_LONG_LONG		(1<<8)
 #define	C_SPEC_SIGNED			(1<<9)
 #define	C_SPEC_UNSIGNED			(1<<10)
-#define	C_SPEC_NATIVE			(1<<11)
-#define	C_SPEC_CONST			(1<<12)
-#define	C_SPEC_VOLATILE			(1<<13)
-#define	C_SPEC_MULTIPLE_BASES	(1<<14)
-#define	C_SPEC_LONG_AND_SHORT	(1<<15)
-#define	C_SPEC_SIGN_AND_UNSIGN	(1<<16)
-#define	C_SPEC_INVALID_COMBO	(1<<17)
-#define	C_SPEC_ENUM				(1<<18)
+#define	C_SPEC_CONST			(1<<11)
+#define	C_SPEC_VOLATILE			(1<<12)
+#define	C_SPEC_MULTIPLE_BASES	(1<<13)
+#define	C_SPEC_LONG_AND_SHORT	(1<<14)
+#define	C_SPEC_SIGN_AND_UNSIGN	(1<<15)
+#define	C_SPEC_INVALID_COMBO	(1<<16)
+#define	C_SPEC_ENUM				(1<<17)
+#define C_SPEC_THREAD_SPECIFIC	(1<<18)
+#define C_SPEC_BOX				(1<<19)
 
 /*
  * Useful specifier combinations.
@@ -57,18 +58,19 @@ extern	"C" {
 			 C_SPEC_STATIC | \
 			 C_SPEC_AUTO | \
 			 C_SPEC_REGISTER | \
-			 C_SPEC_INLINE)
+			 C_SPEC_INLINE | \
+			 C_SPEC_THREAD_SPECIFIC)
 #define	C_SPEC_TYPE_COMMON		\
 			(C_SPEC_CONST | \
 			 C_SPEC_VOLATILE | \
+			 C_SPEC_BOX | \
 			 C_SPEC_ENUM)
 #define	C_SPEC_TYPE_CHANGE		\
 			(C_SPEC_SHORT | \
 			 C_SPEC_LONG | \
 			 C_SPEC_LONG_LONG | \
 			 C_SPEC_SIGNED | \
-			 C_SPEC_UNSIGNED | \
-			 C_SPEC_NATIVE)
+			 C_SPEC_UNSIGNED)
 
 /*
  * Kinds of declarations, for checking the applicability of specifiers.
@@ -165,20 +167,29 @@ typedef struct
 /*
  * Create an array declarator, with unspecified size.
  */
-CDeclarator CDeclCreateOpenArray(ILGenInfo *info, CDeclarator elem);
+CDeclarator CDeclCreateOpenArray(ILGenInfo *info, CDeclarator elem,
+								 int gcSpecifier);
 
 /*
  * Create an array declarator, with a specified size.
  */
-CDeclarator CDeclCreateArray(ILGenInfo *info, CDeclarator elem, ILUInt32 size);
+CDeclarator CDeclCreateArray(ILGenInfo *info, CDeclarator elem,
+							 ILNode *size, int gcSpecifier);
 
 /*
  * Create a pointer declarator.  "refType" will be non-NULL when
- * creating a pointer to another pointer type.
+ * creating a pointer to another pointer/reference type.
  */
 CDeclarator CDeclCreatePointer(ILGenInfo *info, int qualifiers,
 							   CDeclarator *refType);
 
+/*
+ * Create a by-ref '&' declarator.  "refType" will be non-NULL when
+ * creating a reference to another pointer/reference type.
+ */
+CDeclarator CDeclCreateByRef(ILGenInfo *info, int qualifiers,
+							 CDeclarator *refType);
+
 /*
  * Create a function prototype declarator.
  */
