@@ -87,7 +87,9 @@ static char *GetFullClassName(ILClass *classInfo)
 {
 	const char *name = ILClass_Name(classInfo);
 	const char *namespace = ILClass_Namespace(classInfo);
+	char *parentName;
 	char *fullName;
+	ILClass *parent;
 	if(namespace)
 	{
 		fullName = (char *)ILMalloc(strlen(namespace) +
@@ -100,6 +102,19 @@ static char *GetFullClassName(ILClass *classInfo)
 		strcat(fullName, ".");
 		strcat(fullName, name);
 	}
+	else if((parent = ILClass_NestedParent(classInfo)) != 0)
+	{
+		parentName = GetFullClassName(parent);
+		fullName = (char *)ILMalloc(strlen(parentName) + strlen(name) + 2);
+		if(!fullName)
+		{
+			ILDocOutOfMemory(0);
+		}
+		strcpy(fullName, parentName);
+		strcat(fullName, ".");
+		strcat(fullName, name);
+		ILFree(parentName);
+	}
 	else
 	{
 		fullName = (char *)ILMalloc(strlen(name) + 1);
@@ -175,8 +190,6 @@ static char *TypeToName(ILType *type, int shortForm)
 {
 	char *name;
 	ILClass *classInfo;
-	const char *className;
-	const char *classNamespace;
 	const char *suffix;
 	char buffer[128];
 	int posn, kind;
@@ -213,23 +226,13 @@ static char *TypeToName(ILType *type, int shortForm)
 	else if(ILType_IsClass(type) || ILType_IsValueType(type))
 	{
 		classInfo = ILType_ToClass(type);
-		className = ILClass_Name(classInfo);
-		classNamespace = ILClass_Namespace(classInfo);
-		if(!classNamespace || shortForm)
+		if(shortForm)
 		{
-			name = ILDupString(className);
+			name = ILDupString(ILClass_Name(classInfo));
 		}
 		else
 		{
-			name = (char *)ILMalloc(strlen(className) +
-									strlen(classNamespace) + 2);
-			if(!name)
-			{
-				ILDocOutOfMemory(0);
-			}
-			strcpy(name, classNamespace);
-			strcat(name, ".");
-			strcat(name, className);
+			name = GetFullClassName(classInfo);
 		}
 	}
 	else if(type != 0 && ILType_IsComplex(type))
@@ -2680,6 +2683,26 @@ static int ValidateType(FILE *stream, ILContext *context, ILDocType *type)
 	return valid;
 }
 
+/*
+ * Determine if an extra type is known to be a non-standard extra.
+ */
+static int IsNonStandardExtra(ILClass *classInfo)
+{
+	ILAttribute *attr = 0;
+	ILMethod *method;
+	while((attr = ILProgramItemNextAttribute
+				(ILToProgramItem(classInfo), attr)) != 0)
+	{
+		method = ILProgramItemToMethod(ILAttributeTypeAsItem(attr));
+		if(method && !strcmp(ILClass_Name(ILMethod_Owner(method)),
+							 "NonStandardExtraAttribute"))
+		{
+			return 1;
+		}
+	}
+	return 0;
+}
+
 /*
  * Determine if we have an assembly match between a type
  * and an image.
@@ -2717,21 +2740,24 @@ static int CheckForExtraTypes(FILE *stream, ILImage *image,
 			if(!type)
 			{
 				/* This is an extra type */
-				++numExtras;
-				if(xmlOutput)
+				if(!IsNonStandardExtra(classInfo))
 				{
-					fputs("\t<class name=\"", stream);
-					PrintString(fullName, stream);
-					fputs("\" assembly=\"", stream);
-					PrintString(ILImageGetAssemblyName
-						(ILProgramItem_Image(classInfo)), stream);
-					fputs("\">\n", stream);
-					fprintf(stream, "\t\t<extra/>\n");
-					fprintf(stream, "\t</class>\n");
-				}
-				else
-				{
-					fprintf(stream, "%s is not documented\n", fullName);
+					++numExtras;
+					if(xmlOutput)
+					{
+						fputs("\t<class name=\"", stream);
+						PrintString(fullName, stream);
+						fputs("\" assembly=\"", stream);
+						PrintString(ILImageGetAssemblyName
+							(ILProgramItem_Image(classInfo)), stream);
+						fputs("\">\n", stream);
+						fprintf(stream, "\t\t<extra/>\n");
+						fprintf(stream, "\t</class>\n");
+					}
+					else
+					{
+						fprintf(stream, "%s is not documented\n", fullName);
+					}
 				}
 			}
 			else if(!AssemblyMatch(type->assembly, assemName))
