@@ -240,6 +240,9 @@ struct _tagILExecProcess
 	/* Size of the global thread-static allocation */
 	ILUInt32			numThreadStaticSlots;
 
+	/* Image loading flags */
+	int					loadFlags;
+
 #ifdef IL_USE_IMTS
 
 	/* Last-allocated base identifier for interface method tables */
@@ -811,6 +814,16 @@ int _ILExecThreadSelfAborting(ILExecThread *thread);
  */
 int _ILExecThreadHandleWaitResult(ILExecThread *thread, int result);
 
+/*
+ * Gets the state of the thread.
+ */
+ILInt32 _ILExecThreadGetState(ILExecThread *thread, ILThread* supportThread);
+
+/*
+ * Resumes a thread.
+ */
+void _ILExecThreadResumeThread(ILExecThread *thread, ILThread *supportThread);
+
 /*
  * Creates a monitor used by the execution engine.
  * The monitor is created on the GC heap and will be automatically
@@ -829,11 +842,11 @@ int _ILExecMonitorProcessCreate(ILExecProcess *process);
 int _ILExecMonitorProcessDestroy(ILExecProcess *process);
 
 /*
- * Pack a set of arguments into a vararg "Object[]" array.
+ * Pack a set of arguments into a params "Object[]" array.
  * Returns the number of stack words to pop from the function,
  * and the new array in "*array".
  */
-ILUInt32 _ILPackVarArgs(ILExecThread *thread, CVMWord *stacktop,
+ILUInt32 _ILPackCVMStackArgs(ILExecThread *thread, CVMWord *stacktop,
 							ILUInt32 firstParam, ILUInt32 numArgs,
 							ILType *callSiteSig, void **array);
 
