@@ -97,6 +97,17 @@ extern	"C" {
 #define	IL_AF_INET6			23
 #define	IL_AF_IRDA			26
 
+/*
+ * Socket type at the C# level, which may not be the same
+ * as those at the operating system level.
+ */
+#define IL_SOCK_UNKNOWN                 -1
+#define IL_SOCK_STREAM                   1
+#define IL_SOCK_DGRAM                    2
+#define IL_SOCK_RAW                      3
+#define IL_SOCK_RDM                      4
+#define IL_SOCK_SEQPACKET                5 
+
 /*
  * Combined socket address structure.
  */
@@ -367,6 +378,39 @@ ILSysIOHandle ILSysIOSocket(ILInt32 domain, ILInt32 type, ILInt32 protocol)
 		domain = AF_IRDA;
 	}
 #endif
+	
+	switch(type)
+	{
+		case IL_SOCK_STREAM:	
+		{
+			type = SOCK_STREAM; 
+		}
+		break;
+		case IL_SOCK_DGRAM:		
+		{
+			type = SOCK_DGRAM; 
+		}
+		break;
+		case IL_SOCK_RAW:
+		{
+			type = SOCK_RAW; 
+		}
+		break;
+/* BeOS has incomplete socket support */
+#ifdef SOCK_SEQPACKET
+		case IL_SOCK_SEQPACKET:	
+		{
+			type = SOCK_SEQPACKET; 
+		}
+		break;
+#endif
+		default:
+		{
+			type = -1; /* Unknown */
+		}
+		break;
+	}
+	
 	return (ILSysIOHandle)(ILNativeInt)(socket(domain, type, protocol));
 }
 
@@ -624,6 +668,7 @@ ILInt32 ILSysIOSocketSelect(ILSysIOHandle **readfds, ILInt32 numRead,
 			/* Perform a trial select, which may be interrupted */
 			result = select(highest + 1, readPtr, writePtr,
 							exceptPtr, &difftime);
+
 			if(result >= 0 || errno != EINTR)
 			{
 				break;
@@ -653,7 +698,7 @@ ILInt32 ILSysIOSocketSelect(ILSysIOHandle **readfds, ILInt32 numRead,
 							   (struct timeval *)0)) < 0)
 		{
 			/* Keep looping while we are being interrupted by signals */
-			if(result != EINTR)
+			if(errno != EINTR)
 			{
 				break;
 			}
