@@ -36,9 +36,9 @@
  */
 #if !defined(IL_NO_THREADS)
 #if defined(linux) || defined(__linux) || defined(__linux__) || \
-    defined(__FreeBSD__) || defined(__OpenBSD__)
+    defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__sun)
 #if defined(GC_LINUX_THREADS) || defined(GC_FREEBSD_THREADS) || \
-    defined(GC_OPENBSD_THREADS)
+    defined(GC_OPENBSD_THREADS) || defined(GC_SOLARIS_THREADS)
 #define	IL_USE_PTHREADS
 #endif
 #endif
